# Set the hostname of the virtual machine to 'opack' if not specified
OPACK_SYS_HOSTNAME?=	opack

# Set the username for the virtual machine to 'opack' if not specified
OPACK_SYS_USER?=	opack

# Set the username for the virtual machine to 'opack' if not specified
OPACK_SSH_USER?=	$(OPACK_SYS_USER)

# Set the password for the virtual machine to 'opack' if not specified
OPACK_SYS_PASSWORD?=	opack

# Set the disk size of the virtual machine to 4096 MB if not specified
OPACK_SYS_DISK_SIZE?=	4096

# Set the memory (RAM) size of the virtual machine to 512 MB if not specified
OPACK_SYS_MEMORY?=	512

# Set the number of CPUs for the virtual machine to 1 if not specified
OPACK_SYS_CPU?=		1

# Set the OpenBSD package server to 'cdn.openbsd.org' if not specified
OPACK_SYS_SERVER?=cdn.openbsd.org

# Set the architecture for the virtual machine to 'amd64' if not specified
OPACK_SYS_ARCHITECTURE?=amd64

# Automatically determine the OpenBSD release based on the OpenBSD website
OPACK_SYS_RELEASE?=$(shell basename -s .html $(shell curl -s https://www.openbsd.org/ | grep released | cut -d '"' -f 2))

# Extract major and minor version components from the release and construct the version string
OPACK_SYS_VERSION_MAJOR=$(shell echo $(OPACK_SYS_RELEASE) | cut -c 1)
OPACK_SYS_VERSION_MINOR=$(shell echo $(OPACK_SYS_RELEASE) | cut -c 2)
OPACK_SYS_VERSION?=$(OPACK_SYS_VERSION_MAJOR).$(OPACK_SYS_VERSION_MINOR)

# Check if the OpenBSD version is 'snapshots' and automatically increment it if needed
ifeq ($(OPACK_SYS_VERSION), snapshots)
ifeq ($(shell (curl -s https://$(OPACK_SYS_SERVER)/pub/OpenBSD/snapshots/$(OPACK_SYS_ARCHITECTURE)/SHA256 | grep cd$(OPACK_SYS_RELEASE).iso > /dev/null && echo OK) || echo KO), KO)
$(eval OPACK_SYS_RELEASE=$(shell echo $$(($(OPACK_SYS_RELEASE)+1))))
OPACK_SYS_VERSION_MAJOR=$(shell echo $(OPACK_SYS_RELEASE) | cut -c 1)
OPACK_SYS_VERSION_MINOR=$(shell echo $(OPACK_SYS_RELEASE) | cut -c 2)
endif
endif

# Set whether to include Xenocara (X Window System) in the virtual machine to 'no' by default
OPACK_SYS_XENOCARA?=no

# Define the sets of files/packages to install in the virtual machine
OPACK_SYS_SETS?=+* -x* +xbase* -game* -comp* -man* -bsd.rd

# Set the timezone for the virtual machine to 'Europe/Paris' by default
OPACK_SYS_TIMEZONE?=Europe/Paris

# Automatically fetch the Vagrant SSH key
OPACK_SYS_SSH_KEY?=$(shell curl -s https://raw.githubusercontent.com/hashicorp/vagrant/master/keys/vagrant.pub)

# Automatically fetch the latest Packer version
OPACK_PACKER_VERSION?=$$(curl -s https://checkpoint-api.hashicorp.com/v1/check/packer | jq -r .current_version)

# Define the Packer HTTP directory
OPACK_PACKER_HTTP_DIR?=$(OPACK_PACKER_DIR)/http

# Define the URL to fetch the OpenBSD ISO image
OPACK_SYS_ISO_URL?=https://$(OPACK_SYS_SERVER)/pub/OpenBSD/$(OPACK_SYS_VERSION)/$(OPACK_SYS_ARCHITECTURE)/cd$(OPACK_SYS_RELEASE).iso

# Calculate the SHA256 sum of the OpenBSD ISO image
OPACK_SYS_ISO_SHA256SUM?=$(shell curl -s https://$(OPACK_SYS_SERVER)/pub/OpenBSD/$(OPACK_SYS_VERSION)/$(OPACK_SYS_ARCHITECTURE)/SHA256 | grep cd$(OPACK_SYS_RELEASE).iso | cut -d"=" -f2 | tr -d ' ')

# Obtain Git commit information
OPACK_COMMIT?=$(shell git log --oneline -n 1 --abbrev-commit --date=short --pretty=format:"%h %ad %s" 2&>/dev/null || echo latest)
OPACK_SHORT_REV?=$(shell git rev-parse --short HEAD 2&>/dev/null || echo latest)

# Set the location to fetch the sets from to 'http'
OPACK_SYS_SETS_LOCATION?=http

# Set whether to skip file sets signature checking to 'no' by default
OPACK_NO_SIGCHK?=no

# Set whether the virtual machine runs in headless mode to 'true' by default
OPACK_SYS_HEADLESS?=true

# Define the directory for caching
OPACK_CACHE_DIR?=$(OPACK_DIR)boxes

# Define the path to the Vagrant box file and output info
OPACK_BOX_FILE?=$(OPACK_CACHE_DIR)/$(OPACK_TARGET).box

# Define the metadata file for the Vagrant box
OPACK_META_FILE?=$(OPACK_TARGET).json

# Define the version of the Vagrant box
OPACK_BOX_VERSION?=1.$(shell date +%s).$(OPACK_SHORT_REV)

# Define the SHA256 sum for the Vagrant box file
OPACK_BOX_FILE_SHA256?=XXXwillbereplacedXXX

# Define the tag for the Vagrant box
OPACK_BOX_TAG?=opack/$(OPACK_TARGET)

# Define the directory for Packer build
OPACK_PACKER_DIR?=opack_installer_$(OPACK_SHORT_REV)

# Define the provisioning script for Packer
OPACK_PROVISION_FILE?=$(OPACK_DIR)packer-provision.sh

# Define the autodisklabel file
OPACK_AUTODISKLABEL_FILE?=$(OPACK_DIR)autodisklabel

# Define the Vagrantfile
OPACK_VAGRANT_FILE=Vagrantfile

# Define custom networks for Vagrant 
OPACK_VAGRANT_NETWORK?=
#OPACK_VAGRANT_NETWORK=config.vm.network "private_network", auto_config: false

# Define the runtime provisioning script for Vagrant
OPACK_RUNTIME_PROVISION_FILE?=$(OPACK_DIR)vagrant-provision.sh

#OPACK_GCE_JSON_KEY?=skz-mkobsd-92b7a3e770a0.json
#OPACK_GCE_PROJECT?=skz-mkobsd
#OPACK_GCE_BUCKET?=$(GCE_PROJECT)

# OPACK_GCE_PROJECT: The Google Cloud project name where the VM will be created.
OPACK_GCE_PROJECT?=

# OPACK_GCE_JSON_KEY: The filename of the Google Cloud service account JSON key.
OPACK_GCE_JSON_KEY?=

# OPACK_GCE_BUCKET: The Google Cloud Storage bucket name, typically associated with the project, for the image import.
OPACK_GCE_BUCKET?=$(OPACK_GCE_PROJECT)

# OPACK_GCE_REGION: The Google Cloud Compute Engine region.
OPACK_GCE_REGION?=europe-west4

# OPACK_GCE_REGION: The Google Cloud Compute Engine zone.
OPACK_GCE_REGION?=europe-west4-b

# Define the machine type for the VM
OPACK_GCE_MACHINE?=e2-micro

# The SSH key to use to connect to the cloud VM
OPACK_SYS_SSH_PRIVATE_KEY=id_ed25519
OPACK_SYS_SSH_PUBLIC_KEY=$(shell cat $(OPACK_SYS_SSH_PRIVATE_KEY).pub)

# Define the directory for Terraform deployment
OPACK_TERRAFORM_DIR?=opack_deploy_$(OPACK_SHORT_REV)

# Terraform module dir
OPACK_TERRAFORM_MODULE_DIR?=$(OPACK_TERRAFORM_DIR)/opack-module

ifndef OPACK_DEBUG
$(info ░ BOX        $(OPACK_BOX_FILE))
$(info ▒ HOSTNAME   $(OPACK_SYS_HOSTNAME))
$(info ░ USER       $(OPACK_SYS_USER))
$(info ▒ PASSWORD   $(OPACK_SYS_PASSWORD))
$(info ░ DISK-SIZE  $(OPACK_SYS_DISK_SIZE)m)
$(info ▒ ARCH       $(OPACK_SYS_ARCHITECTURE))
$(info ░ CPU COUNT  $(OPACK_SYS_CPU))
$(info ▒ MEMORY     $(OPACK_SYS_MEMORY)m)
$(info ░ RELEASE    $(OPACK_SYS_RELEASE))
$(info ▒ VERSION    $(OPACK_SYS_VERSION))
$(info ░ SERVER     $(OPACK_SYS_SERVER))
$(info ▒ XENOCARA   $(OPACK_SYS_XENOCARA))
$(info ░ SETS       $(OPACK_SYS_SETS))
$(info ▒ IGNORE-SIG $(OPACK_NO_SIGCHK))
$(info ░ HEADLESS   $(OPACK_SYS_HEADLESS))
endif
